/// Listening keyboard input from barcode scanner
///
/// Most of specialized barcode scanners can return
/// result as an imitation of keyboard input, this library let
/// get the result of this input
library barcode_keyboard_listener;

import 'package:flutter/services.dart';
import 'dart:convert';

/// Callback to return scanning result
typedef ScanCallback = void Function(String scanString);

/// Callback for reaction on button's pushing
typedef KeyCodeCallback = void Function(int keyCode);

/// Listening keyboard input from barcode scanner
///
/// Listens to keyboard input. Upon detection of scanning button press,
/// records incoming sequence of presses, after the scanning button was released
/// provides recorded data as a string of scanning result
class BarcodeListener{
  
  /// activate/deactivate reaction to events incoming from keyboard
  bool enabled = false;

  final ScanCallback _scanCallback;
  final int _scanButtonKeyCode;
  final List<int> _otherKeyCodeList;
  bool _scanning = false;
  final _bcCodeUnits = List<int>();
  final KeyCodeCallback _otherKeyCallback;

  /// parameters:
  /// * [_scanCallback]      - required, callback to return results of scanning 
  /// * [_scanButtonKeyCode] - required, code for scanning button
  /// * [_otherKeyCallback]  - callback for reaction to "others" buttons press
  /// * [_otherKeyCodeList]  - list of codes of "others" buttons for which will be called<br>
  /// callback from a parameter [_otherKeyCallback]
  BarcodeListener(this._scanCallback, this._scanButtonKeyCode, [this._otherKeyCallback, this._otherKeyCodeList]) {
    RawKeyboard.instance.addListener(_keyBoardCallback);
    enabled = true;
    _scanning = false;
  }

  _keyBoardCallback(RawKeyEvent keyEvent){
    if (!enabled) return;

    var data = keyEvent.data as RawKeyEventDataAndroid;

    if(keyEvent is RawKeyDownEvent && data.keyCode == _scanButtonKeyCode) {
      _scanning = true;
      return;
    }

    if(!_scanning && keyEvent is RawKeyDownEvent && _otherKeyCallback != null) {
      if (
        (_otherKeyCodeList != null && _otherKeyCodeList.contains(data.keyCode)) ||
        (_otherKeyCodeList == null)
      ){
        _otherKeyCallback(data.keyCode);
        return;
      }
    }

    if (!_scanning) return;

    if(keyEvent is RawKeyUpEvent && data.keyCode == _scanButtonKeyCode) {
      if (this._bcCodeUnits.isNotEmpty) {
        final barcode = utf8.decode(this._bcCodeUnits);
        _scanCallback(barcode);
        this._bcCodeUnits.clear();
      }
      _scanning = false;
      return;
    }

    if(keyEvent is RawKeyDownEvent && data.codePoint != 0 && data.codePoint != 10) {
      _bcCodeUnits.add(data.codePoint);
    }
  }

  /// release of used resources
  void dispose(){
    RawKeyboard.instance.removeListener(_keyBoardCallback);
  }
}